﻿const config = require('config.json');
const jwt = require('jsonwebtoken');
const bcrypt = require('bcryptjs');
const db = require('_helpers/db');
const sendEmail = require('lib/send-email');

module.exports = {
    authenticate,
    getAll,
    getById,
    create,
    update,
    delete: _delete
};

async function authenticate({ email, password }) {
    const gamer = await db.Gamer.scope('withHash').findOne({ where: { email } });

    if (!gamer || !(await bcrypt.compare(password, gamer.hash)))
        throw 'email or password is incorrect';

    console.log('gamer: ' + JSON.stringify(gamer));
        
    // authentication successful
    const token = jwt.sign({ sub: gamer.id }, config.secret, { expiresIn: '7d' });
    return { ...omitHash(basicDetails(gamer)), token };
}

async function getAll() {
    return await db.Gamer.findAll();
}

async function getById(id) {
    return await getGamer(id);
}

async function create(params) {
    // validate
    if (await db.Gamer.findOne({ where: { email: params.email } })) {
        throw 'Email "' + params.email + '" already registered';
    }

    // hash password
    if (params.password) {
        params.hash = await bcrypt.hash(params.password, 10);
    }

    // save Gamer
    await db.Gamer.create(params);

     // send email
     await sendVerificationEmail(params);  
}


async function sendVerificationEmail(gamer) {

    await sendEmail({
        to: gamer.email,
        subject: 'Thansk for Sign-up',
        html: `<p>Thanks for registering!</p>`
    });
}

async function update(id, params) {
    const gamer = await getGamer(id);

    // validate
    const emailChanged = params.email && gamer.email !== params.email;
    if (emailChanged && await db.Gamer.findOne({ where: { email: params.email } })) {
        throw 'Email "' + params.email + '" is already taken';
    }

    // hash password if it was entered
    if (params.password) {
        params.hash = await bcrypt.hash(params.password, 10);
    }

    // copy params to gamer and save
    Object.assign(gamer, params);
    await gamer.save();

    return omitHash(gamer.get());
}

async function _delete(id) {
    const gamer = await getGamer(id);
    await gamer.destroy();
}

// helper functions

async function getGamer(id) {
    const gamer = await db.Gamer.findByPk(id);
    if (!gamer) throw 'Gamer not found';
    return gamer;
}

function omitHash(gamer) {
    const { hash, ...userWithoutHash } = gamer;
    return userWithoutHash;
}

function basicDetails(gamer) {
    const { firstName, lastName, email, createdBy, updatedBy } = gamer;
    return { firstName, lastName, email, createdBy, updatedBy };
}